@@ -7,6 +7,7 @@ import sys
 import os
 from pathlib import Path
 from pprint import pformat
+import time
 
 import datetime
 import logging
@@ -490,7 +491,7 @@ class Terminal(QtWidgets.QMainWindow):
     # Listens & inter-object methods
 
     def ping_pilot(self, pilot):
-        self.send(pilot, 'PING')
+        self.node.send(pilot, 'PING')
 
     def heartbeat(self, once=False):
         """
@@ -596,6 +597,9 @@ class Terminal(QtWidgets.QMainWindow):
             task = self.subjects[subject_name].prepare_run()
             task['pilot'] = value['pilot']
 
+            # FIXME: Don't hardcode wait time, wait until we get confirmation that the running task has fully unloaded
+            time.sleep(5)
+
             self.node.send(to=value['pilot'], key="START", value=task)
 
     def l_ping(self, value):
