@@ -4,32 +4,22 @@ import sys
 import os
 import csv
 from skvideo import io
-from skvideo.utils import vshape
-import numpy as np
-import base64
 from datetime import datetime
 import multiprocessing as mp
 from tqdm.auto import tqdm
-
+import inspect
+import typing
 
 import time
 import traceback
 import blosc
 import warnings
 import subprocess
+
+from queue import Queue, Empty, Full
 import logging
 from ctypes import c_char_p
-
-
-
-
-# import the Queue class from Python 3
-if sys.version_info >= (3, 0):
-    from queue import Queue, Empty, Full
-
-# otherwise, import the Queue class for Python 2.7
-else:
-    from Queue import Queue, Empty, Full
+import numpy as np
 
 try:
     import PySpin
@@ -44,6 +34,12 @@ try:
 except:
     OPENCV = False
 
+try:
+    import picamera
+    PICAMERA = True
+except:
+    PICAMERA = False
+
 from autopilot import prefs
 from autopilot.hardware import Hardware
 
@@ -66,6 +62,10 @@ class Camera(Hardware):
     Arguments:
         fps (int): Framerate of video capture
         timed (bool, int, float): If False (default), camera captures indefinitely. If int or float, captures for this many seconds
+        rotate (int): Number of times to rotate image clockwise (default 0). Note that
+            image rotation should happen in :meth:`._grab` or be otherwise implemented
+            in each camera subclass, because it's a common enough operation many
+            cameras have some optimized way of doing it.
         **kwargs: Arguments to :meth:`~Camera.stream`, :meth:`~Camera.write`, and :meth:`~Camera.queue` can be passed as dictionaries, eg.::
 
             stream={'to':'T', 'ip':'localhost'}
@@ -114,7 +114,7 @@ class Camera(Hardware):
     type = "CAMERA" #: (str): what are we anyway?
     trigger = False
 
-    def __init__(self, fps=None, timed=False, crop=None, **kwargs):
+    def __init__(self, fps=None, timed=False, crop=None, rotate:int=0, **kwargs):
         """
 
         Args:
@@ -133,11 +133,13 @@ class Camera(Hardware):
         self._write_q = None
         self._stream_q = None
         self._indicator = None
+        self._resolution = None
 
         self.frame = None
         self.shape = None
         self.frame_n = 0
         self.crop = crop
+        self.rotate = rotate
 
         self.blosc = True
 
@@ -183,7 +185,6 @@ class Camera(Hardware):
         """
         Spawn a thread to begin capturing.
 
-
         Args:
             timed (None, int, float): if None, record according to :attr:`.timed` (default). If numeric, record for ``timed`` seconds.
         """
@@ -217,11 +218,6 @@ class Camera(Hardware):
         * :meth:`~.Camera._write_frame`  - how to write an individual frame to disk
         * :meth:`~.Camera._write_deinit` - any required routine to finish writing to disk after acquisition
         * :meth:`~.Camera.capture_deinit` - any required routine to stop acquisition but not release the camera instance.
-
-
-
-        Returns:
-
         """
 
         self.capturing.set()
@@ -254,7 +250,7 @@ class Camera(Hardware):
             try:
                 if self.streaming.is_set():
                     self.node.send(key='STATE', value='STOPPING')
-                    self._stream_q.put('END')
+                    self._stream_q.append('END')
             except Exception as e:
                 self.logger.exception('Failed to end stream, error message: {}'.format(e))
 
@@ -291,8 +287,11 @@ class Camera(Hardware):
             self.logger.exception(e)
 
         if self.streaming.is_set():
-            self._stream_q.put_nowait({'timestamp': self.frame[0],
-                                       self.name  : self.frame[1]})
+            try:
+                self._stream_q.append({'timestamp': self.frame[0],
+                                           self.name  : self.frame[1]})
+            except Full:
+                self.logger.exception(f"queue was full for frame captured at {self.frame[0]}")
 
         if self.writing.is_set():
             self._write_frame()
@@ -318,6 +317,8 @@ class Camera(Hardware):
             to (str): ID of the recipient. Default 'T' for Terminal.
             ip (str): IP of recipient. If None (default), 'localhost'. If None and ``to`` is 'T', ``prefs.get('TERMINALIP')``
             port (int, str): Port of recipient socket. If None (default), ``prefs.get('MSGPORT')``. If None and ``to`` is 'T', ``prefs.get('TERMINALPORT')``.
+            min_size (int): Number of frames to collect before sending (default: 5). use 1 to send frames as soon as they are available,
+                sacrificing the efficiency from compressing multiple frames together
             **kwargs: passed to :meth:`.Hardware.init_networking` and thus to :class:`.Net_Node`
 
         """
@@ -393,7 +394,7 @@ class Camera(Hardware):
                 if False, timestamps will be generated by :class:`.Video_Writer` (not recommended at all).
             blosc (bool): if true, compress frames with :func:`blosc.pack_array` before putting in :attr:`._write_q`.
         """
-        if not output_filename:
+        if output_filename is None:
             output_filename = self.output_filename
         else:
             self._output_filename = output_filename
@@ -498,6 +499,22 @@ class Camera(Hardware):
     def output_filename(self, output_filename):
         self._output_filename = output_filename
 
+    @property
+    def resolution(self):
+        return self._resolution
+
+    @resolution.setter
+    def resolution(self, resolution):
+        self._resolution = resolution
+
+    @property
+    def fps(self):
+        return self._fps
+
+    @fps.setter
+    def fps(self, fps):
+        self._fps = fps
+
     def _grab(self):
         """
         Capture a frame and timestamp.
@@ -574,6 +591,296 @@ class Camera(Hardware):
         # raise Exception('release must be overwritten by camera subclass!!')
 
 
+class PiCamera(Camera):
+    """
+    Interface to the `Raspberry Pi Camera Module <https://www.raspberrypi.org/products/camera-module-v2/>`_
+    via `picamera <https://picamera.readthedocs.io/en/latest/>`_
+
+    Parameters of the :class:`picamera.PiCamera` class can be set after initialization by modifying the
+    :attr:`PiCamera.cam` attribute, eg ``PiCamera().cam.exposure_mode = 'fixedfps'`` -- see the
+    :class:`picamera.PiCamera` documentation for full documentation.
+
+    Note that some parameters, like resolution, can't be changed after starting :meth:`~PiCamera.capture` .
+
+    The Camera Module is a slippery little thing, and ``fps`` and ``resolution`` are just requests
+    to the camera, and aren't necessarily followed with 100% fidelity. The possible framerates and resolutions
+    are determined by the ``sensor_mode`` parameter, which by default tries to guess the best
+    sensor mode based on the fps and resolution. See the :ref:`picamera:camera_modes` documentation for
+    more details.
+
+    This wrapper uses a subclass, :class:`PiCamera.PiCamera_Writer` to capture frames decoded by the
+    gpu directly from the preallocated buffer object. Currently the restoration from the buffer
+    assumes that RGB, or generally ``shape[2] == 3``, images are being captured. See
+    `this stackexchange post <https://raspberrypi.stackexchange.com/a/58941/112948>`_ by Dave Jones, author
+    of the picamera module, for a strategy for capturing grayscale images quickly.
+
+    This class also currently uses the default :class:`Video_Writer` object, but it could be
+    more performant to use the :meth:`picamera.PiCamera.start_recording` method's built-in ability
+    to record video to a file --- try it out!
+
+    .. todo::
+
+        Currently timestamps are constructed with :meth:`datetime.datetime.now.isoformat`, which is
+        not altogether accurate. Timestamps should be gotten from the :attr:`~picamera.PiCamera.frame` attribute,
+        which depends on the :attr:`~picamera.PiCamera.clock_mode`
+
+
+    References:
+        - https://blog.robertelder.org/recording-660-fps-on-raspberry-pi-camera/
+        - Fast capture from the author of picamera - https://raspberrypi.stackexchange.com/a/58941/112948
+        - More on fast capture and processing, see last example in section - https://picamera.readthedocs.io/en/release-1.12/recipes2.html#rapid-capture
+    """
+
+    def __init__(self, camera_idx:int=0, sensor_mode:int=0,
+                 resolution:typing.Tuple[int,int] = (1280,720),
+                 fps:int=30,
+                 format:str='rgb',
+                 *args, **kwargs):
+        """
+        Args:
+            camera_idx (int): Index of picamera (default: 0, >=1 only supported on compute module)
+            sensor_mode (int): Sensor mode, default 0 detects automatically from resolution and fps,
+                note that sensor_mode will affect the available resolutions and framerates,
+                see :ref:`picamera:camera_modes` for more information
+            resolution (tuple): a tuple of (width, height) integers, but mind the note in the above documentation
+                regarding the sensor_mode property and resolution
+            fps (int): frames per second, but again mind the note on sensor_mode
+            format (str): Format passed to :class`picamera.PiCamera.start_recording` one of ``('rgb' (default), 'grayscale')``
+                The ``'grayscale'`` format uses the ``'yuv'`` format, and extracts the luminance channel
+            *args (): passed to superclass
+            **kwargs (): passed to superclass
+        """
+        super(PiCamera, self).__init__(*args, **kwargs)
+
+        if not globals()['PICAMERA']:
+            self.logger.exception('the picamera package could not be imported, install it before use!')
+            return
+
+        self._sensor_mode = None
+        self._cam = None
+        self._picam_writer = None
+
+        self.camera_idx = camera_idx
+        self.sensor_mode = sensor_mode
+        self.resolution = resolution
+        self.fps = fps
+        self.format = format
+        self.rotation = self.rotate * 90
+
+    @property
+    def sensor_mode(self) -> int:
+        """
+        Sensor mode, default 0 detects automatically from resolution and fps,
+        note that sensor_mode will affect the available resolutions and framerates,
+        see :ref:`picamera:camera_modes` for more information.
+
+        When set, if the camera has been initialized, will change the attribute in :attr:`PiCamera.cam`
+
+        Returns:
+            int
+        """
+        return self._sensor_mode
+
+    @sensor_mode.setter
+    def sensor_mode(self, sensor_mode: int):
+        self._sensor_mode = sensor_mode
+        if self.initialized.is_set():
+            self.cam.sensor_mode = self._sensor_mode
+
+    @property
+    def resolution(self) -> typing.Tuple[int, int]:
+        """
+        A tuple of ints, (width, height).
+
+        Resolution can't be changed while the camera is capturing.
+
+        See :ref:`picamera:camera_modes` for more information re: how resolution relates to
+        :attr:`picamera.PiCamera.sensor_mode`
+
+        Returns:
+            tuple of ints, (width, height)
+        """
+        return self._resolution
+
+    @resolution.setter
+    def resolution(self, resolution: typing.Tuple[int, int]):
+        self._resolution = resolution
+        if self._picam_writer is not None:
+            self._picam_writer.resolution = self._resolution
+        if self.initialized.is_set() and not self.capturing.is_set():
+            self.cam.resolution = self._resolution
+        elif self.capturing.is_set():
+            self.logger.warning('cant set resolution while camera is capturing!')
+
+    @property
+    def fps(self) -> int:
+        """
+        Frames per second
+
+        See :ref:`picamera:camera_modes` for more information re: how fps relates to
+        :attr:`picamera.PiCamera.sensor_mode`
+
+        Returns:
+            int - fps
+        """
+        return self._fps
+
+    @fps.setter
+    def fps(self, fps):
+        self._fps = fps
+        if self.initialized.is_set():
+            self.cam.framerate = self._fps
+
+    @property
+    def rotation(self) -> int:
+        """
+        Rotation of the captured image, derived from :attr:`.Camera.rotate` * 90.
+
+        Must be one of ``(0, 90, 180, 270)``
+
+        Rotation can be changed during capture
+
+        Returns:
+            int - Current rotation
+        """
+        return self._rotation
+
+    @rotation.setter
+    def rotation(self, rotation:int):
+        rotation = int(round(rotation))
+        if rotation not in (0, 90, 180, 270):
+            errmsg = f"rotation must be 0, 90, 180, or 270, got {rotation}"
+            self.logger.exception(errmsg)
+            raise ValueError(errmsg)
+
+        self._rotation = rotation
+        if self.initialized.is_set():
+            self.cam.rotation = self._rotation
+
+    def init_cam(self) -> 'picamera.PiCamera':
+        """
+        Initialize and return the :class:`picamera.PiCamera` object.
+
+        Uses the stored :attr:`~PiCamera.camera_idx`, :attr:`~PiCamera.resolution`,
+        :attr:`~PiCamera.fps`, and :attr:`~PiCamera.sensor_mode` attributes on init.
+
+        Returns:
+            :class:`picamera.PiCamera`
+        """
+
+        cam = picamera.PiCamera(
+            camera_num=self.camera_idx,
+            resolution=self.resolution,
+            framerate=self.fps,
+            sensor_mode=self.sensor_mode,
+        )
+        cam.rotation = self._rotation
+
+        self.initialized.set()
+
+        return cam
+
+    def capture_init(self):
+        """
+        Spawn a :class:`PiCamera.PiCamera_Writer` object to :attr:`PiCamera._picam_writer`
+        and :meth:`~picamera.PiCamera.start_recording` in the set :attr:`~PiCamera.format`
+        """
+        self._picam_writer = self.PiCamera_Writer(self.resolution, self.format)
+        format = self.format
+        if format == "grayscale":
+            format = 'yuv'
+
+        self.cam.start_recording(self._picam_writer, format)
+
+    def _grab(self) -> typing.Tuple[str, np.ndarray]:
+        """
+        Wait on the :attr:`~PiCamera.PiCamera_Writer.grab_event` to be set,
+        then clear it before returning the frame.
+
+        Returns:
+            (timestamp, frame) tuple
+        """
+        # wait until a new frame is captured
+        self._picam_writer.grab_event.wait()
+        ret = (self._picam_writer.timestamp, self._picam_writer.frame)
+        self._picam_writer.grab_event.clear()
+        return ret
+
+    def capture_deinit(self):
+        """
+        :meth:`~picamera.PiCamera.stop_recording` and :meth:`~picamera.PiCamera.close` the camera,
+        releasing its resources.
+        """
+        self.cam.stop_recording()
+        # self.cam.close()
+
+    def release(self):
+        self._picam_writer.grab_event.clear()
+        super(PiCamera, self).release()
+        self._picam_writer.grab_event.clear()
+        try:
+            self.cam.close()
+        except KeyError as e:
+            self.logger.debug(f"Exception closing picamera: {e}")
+
+        self._cam = None
+
+    class PiCamera_Writer(object):
+        """
+        Writer object for processing individual frames,
+        see: https://raspberrypi.stackexchange.com/a/58941/112948
+        """
+        def __init__(self, resolution:typing.Tuple[int, int], format:str="rgb"):
+            """
+            Args:
+                resolution (tuple): (width, height) tuple used when making numpy array from buffer
+
+            Attributes:
+                grab_event (:class:`threading.Event`): Event set whenever a new frame is captured,
+                    cleared by the parent class when the frame is consumed.
+                frame (:class:`numpy.ndarray`): Captured frame
+                timestamp (str): Isoformatted timestamp of time of capture.
+
+            """
+            self.resolution = resolution
+            self._block_resolution = (
+                self.resolution[0]+ 31 // 32 * 32,
+                self.resolution[1] + 15 // 16 * 16
+            )
+            self.format = format
+            self.grab_event = threading.Event()
+            self.grab_event.clear()
+            self.frame = None
+            self.timestamp = None
+
+        def write(self, buf):
+            """
+            Reconstutute the buffer into a numpy array in :attr:`PiCamera_Writer.frame` and
+            make a timestamp in :attr:`PiCamera_Writer.timestamp`, then set the :attr:`PiCamera_Writer.grab_event`
+
+            Args:
+                buf (): Buffer given by PiCamera
+            """
+            if self.format == 'grayscale':
+                # just capture the luminance channel. see
+                # https://raspberrypi.stackexchange.com/a/58941/112948
+                # and
+                # https://raspberrypi.stackexchange.com/a/58941/112948
+                self.frame = np.frombuffer(
+                    buf, dtype=np.uint8,
+                    count=self._block_resolution[0]*self._block_resolution[1]
+                ).reshape((self._block_resolution[1],self._block_resolution[0]))[:self.resolution[1], :self.resolution[0]]
+            else:
+                self.frame = np.frombuffer(
+                    buf, dtype=np.uint8,
+                    count=self.resolution[0]*self.resolution[1]*3
+                ).reshape((self.resolution[1], self.resolution[0], 3))
+            self.timestamp = datetime.now().isoformat()
+            self.grab_event.set()
+
+
+
+
 
 
 
@@ -672,7 +979,8 @@ class Camera_CV(Camera):
         ts = self._timestamp()
         if self.crop:
             frame = frame[self.crop[1]:self.crop[1]+self.crop[3], self.crop[0]:self.crop[0]+self.crop[2]]
-        return (ts, frame)
+
+        return (ts, np.rot90(frame, axes=(1,0), k=self.rotate))
 
     def _timestamp(self, frame=None):
         """
@@ -1007,13 +1315,13 @@ class Camera_Spinnaker(Camera):
 
         if self.streaming.is_set():
             if not frame_array:
-                frame_array = self.frame[1].GetNDArray()
-            self._stream_q.put_nowait({'timestamp': self.frame[0],
+                frame_array = np.rot90(self.frame[1].GetNDArray(), axes=(1,0), k=self.rotate)
+            self._stream_q.append({'timestamp': self.frame[0],
                                        self.name  : frame_array})
 
         if self.queueing.is_set():
             if not frame_array:
-                frame_array = self.frame[1].GetNDArray()
+                frame_array = np.rot90(self.frame[1].GetNDArray(), axes=(1,0), k=self.rotate)
             self.q.put_nowait((self.frame[0], frame_array))
 
         if self.indicating.is_set():
@@ -1687,6 +1995,7 @@ class Video_Writer(mp.Process):
                     break
 
         finally:
+            vid_out.close()
 
             # save timestamps as .csv
             ts_path = os.path.splitext(self.path)[0] + '.csv'
@@ -1695,7 +2004,6 @@ class Video_Writer(mp.Process):
                 for ts in self.timestamps:
                     csv_writer.writerow([ts])
 
-            vid_out.close()
 
 
 def list_spinnaker_cameras():
